/*
    Jeu de "ptages" de Ballons,
    illustration de cration d'objets et d'affichage graphique slectif

*/

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;

public class PetageDeBalounes extends Applet
    implements MouseListener, ItemListener{    
    private int nb, nbDebut,                // nombre de balounes
        xDebut, yDebut, xFin, yFin;         // dimension de la fentre
    
    private Effacable[] b;                    // tableau des Balounes
    // temps du dbut d'execution et de la derniere Baloune crevee
    private long tempsDebut,tempsClic;  
    
    private Graphics g;  // ncessaire pour l'affichage rapide
              
    private int affichage = 2; // mode d'affichage 0,1 ou 2
    private CheckboxGroup c;
    private Checkbox [] cb = new Checkbox[3];
    private String [] cbn = {"Lent","Moyen","Rapide"};
    private Color [] couleurs = {Color.red,Color.green,Color.blue,Color.yellow,
                                 Color.cyan, Color.magenta,Color.orange};
    
    // donne un entier au hasard dans l'intervalle [a,b]
    private int hasard(int a, int b){
        return (int)(Math.random()*(b-a+1))+a;
    }
    
    public void init(){
        // crer le choix d'affichage
        c = new CheckboxGroup();
        for (int i=0;i<3;i++){
            cb[i]=new Checkbox(cbn[i],c,affichage==i);
            add(cb[i]);
            cb[i].addItemListener(this);
        }
        // crer l'affichage
        nb = 20;
        nbDebut = nb;
        b = new Effacable[nb];
        xDebut = 0; yDebut=20;
        xFin = getSize().width;  
        yFin = getSize().height; 
        System.out.println(xFin+","+yFin);

        g = getGraphics();
        Effacable.setFondEcran(getBackground());
        // creation des Ballons
        for(int k=0;k<nb;k++){
            b[k]=new Effacable(hasard(xDebut+10,xFin-10),
                                hasard(yDebut+10,yFin-10),
                                hasard(30,50),
                                couleurs[hasard(0,couleurs.length-1)]);
            System.out.println(k+" = "+b[k]);
        }
        repaint(); // les afficher
        // indiquer qu'on va dtecter les clics de souris
        addMouseListener(this);
        // conserver la valeur du dbut de l'excution
        tempsDebut = System.currentTimeMillis();
    }
    
    // afficher le temps d'execution 
    private void messageFin(Graphics g){
        float temps = (tempsClic-tempsDebut)/1000.0f;
        g.drawString("Vous avez crev toutes les Balounes en ",10,100);
        g.drawString(temps+" secondes",30,120);
        if (temps < nbDebut/2) g.drawString("Trs rapide",30,150);
        else if (temps < nbDebut) g.drawString("Assez rapide",30,150);
        else g.drawString("Pas vite!",30,150);
    }
        
    public void paint( Graphics g ) {
        if(nb>0)
            for (int i=0;i<nb;i++)b[i].afficher(g,"");
        else messageFin(g);
    }
    
    // traitement du choix de l'affichage
    public void itemStateChanged(ItemEvent e){
        for (int i=0; i<3;i++)
            if(e.getSource()==cb[i]){
                affichage=i;
                break;
            }
    }
    
    // traitement des clics de souris
    public void mouseClicked(MouseEvent event){
    }
    
    public void mouseReleased(MouseEvent event){
    }
    
    public void mousePressed(MouseEvent event){
        int x = event.getX();
        int y = event.getY();
        if(nb>0){
            tempsClic=System.currentTimeMillis();
            for(int i=nb-1;i>=0;i--)
                if(b[i].contient(x,y)){
                    System.out.println("Crve "+ i);
                    Effacable crevee=b[i];
                    nb--;
                    if(nb>0){
                        // enlever la Baloune creve
                        for(int j=i;j<nb;j++) b[j]=b[j+1];
                        switch (affichage){
                            // raffiche tout l'cran
                        case 0 : repaint();  
                            break;
                            // ne raffiche que la rgion autour de la
                            // Baloune creve
                        case 1 : repaint(crevee.gauche(),crevee.haut(),
                                         crevee.largeur(),crevee.hauteur());
                        break;
                        // efface la Baloune et ne redessine que ce qui
                        // lui touche sans jamais mettre de rgion  blanc 
                        // ce qui vite le clignotement
                        case 2 : crevee.effacer(g);
                            for(int j=0;j<nb;j++)
                                if(b[j].touche(crevee)){
                                    b[j].afficher(g,"");
                                }
                            break;
                        }
                    } else repaint();
                    break;
                }
        }
    }
    
    public void mouseEntered(MouseEvent event){
    }
    
    public void mouseExited(MouseEvent event){
    }
    

}
